/***************************************************************************/
// PRACTICA 5 DE ALGORITMOS Y ESTRUCTURAS DE DATOS
//
/* Programa para la gestion de los test de autoescuela                      */
/* Curso 2004-2005                                                             */
//
// Ariadna Fuertes
// Jorge Esteve Taboada
//
/***************************************************************************/

#include <iostream>
#include <fstream>
#include <string>

#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#include "lista2cabeza.h"

using namespace std;

// Declaraciones de funciones
bool LeerFich(Lista &);
int Menu();
void GenerarTest(Lista &);
void IntroducePregunta(Lista&);
bool BuscaPreguntaPorID (Lista&, int);
bool SumaUnaVez (Lista&);
void EliminaPregunta(Lista&);
void MostrarLista(Lista);
void EscribeFich(Lista&);
void EscribeTest(Lista&);
void MuestraPregunta(Valor);
void MovimientoAleatorio(Lista&, int);
void HallaMediaDif(Lista);
void HallaNumPregUsadas(Lista);
void HallaNumPregNoUsadas(Lista);
void MostrarPreguntasPorNivel(Lista);

// **********************************************************************
// *******************************  MAIN  *******************************
// **********************************************************************
int main()
{
    bool error;
    int opc;
    string texto;
    Lista almacen;

    //inicializar semilla de numeros aleatorios
    srand(time(NULL));

    // Cargamos las preguntas desde el fichero "preguntas.txt" a la lista "almacen"
    error = LeerFich(almacen);

    if (!error)
    {
         cout << "--------------------------------------------------------------" << endl;
         cout << "Preguntas importadas correctamente del fichero 'preguntas.txt'" << endl;
         cout << "--------------------------------------------------------------" << endl;
    }

    do
    {
        if (!error) // Mostramos menu de opciones
           opc = Menu();
        else
           opc = 0; // Salir

        cout << endl;

        switch(opc)
        {
         case 1:   // Generar Test
                   GenerarTest(almacen);
                   break;
         case 2:   // Introduce nueva pregunta
                   IntroducePregunta(almacen);
                   break;
         case 3:   // Elimina pregunta
                   EliminaPregunta(almacen);
                   break;
         case 4:   // Mostrar preguntas
                   MostrarLista(almacen);
                   break;
         case 5:   // Muestra todas las preguntas por pantalla
                   EscribeFich(almacen);
                   break;
                   // OPCIONALES
         case 6:   // Halla y muestra la media de dificultad de las preguntas
                   HallaMediaDif(almacen);
                   break;
         case 7:   // Halla el nmero de preguntas usadas alguna vez
                   HallaNumPregUsadas(almacen);
                   break;
         case 8:   // Halla el nmero de preguntas no usadas nunca
                   HallaNumPregNoUsadas(almacen);
                   break;
         case 9:   // Muestra todas las preguntas por pantalla
                   MostrarPreguntasPorNivel(almacen);
                   break;
        }
    } while(opc!=0);
} 


/******************************************************************************/
/* LeerFich     lee la lista de preguntas disponibles en el fichero           */
/* parametros E/S                                                             */
/* almacen      S     lista de preguntas leida                                */
/* error        S     si el fichero esta vacio o no se ha podido leer da true */
/******************************************************************************/
bool LeerFich(Lista & l)
{
   ifstream fich;
   Valor t;
   bool error=false;
   int id = 1;

   fich.open("preguntas.txt");
   if(!fich)
   {
       cerr << "Error de apertura de fichero 'preguntas.txt'\n";
       error=true;
   }
   else
   {
       while(getline(fich , t.pregunta))
       {
           getline(fich , t.resA);
           getline(fich , t.resB);
           getline(fich , t.resC);
           fich >> t.correcta;
           fich >> t.nivel;
           fich >> t.veces;
           t.ID = id;
           fich.ignore();

           // Avanzamos el identificador de pregunta
           id = id + 1;
           
           l.Insertar(t);
       }
       fich.close();
   }
   return error;        
}


/******************************************************************************/
/* Menu      muestra en pantalla el menu de opciones                          */
/* devuelve la opcion elegida                                                 */
/******************************************************************************/
int Menu(void)
{
   int opcion;
   
   do {
      cout << endl << "================================" << endl;
      cout << "Generador de Test de autoescuela" << endl;
      cout << "================================" << endl << endl;
      cout << "1.- Generar nuevo test" << endl;
      cout << "2.- Incorporar nueva pregunta" << endl;
      cout << "3.- Eliminar pregunta" << endl;
      cout << "4.- Mostrar todas las preguntas" << endl;
      cout << "5.- Guardar todas las preguntas en fichero 'preguntas.txt'" << endl << endl;

      cout << "6.- Consultar la media de dificultad del total de preguntas" << endl;
      cout << "7.- Hallar el numero de preguntas usadas alguna vez" << endl;
      cout << "8.- Hallar el numero de preguntas no usadas" << endl;
      cout << "9.- Mostrar preguntas de un nivel concreto" << endl << endl;

      cout << "0.- Salir del programa" << endl << endl;

      cout << "Elija opcion: ";
      cin >> opcion;
   }
   while ((opcion < 0) || (opcion > 9));

   return opcion;
}




/* *******************************************************************
   Genera un test a partir de las preguntas de la lista 'l'.
   Se realiza una selecci� de las preguntas atendiendo al nmero
   de veces que se repiten las preguntas.
   Posteriormente se escogen aleatoriamente las que formar� el test.
   Se muestra por pantalla el test y se extrae a los ficheros .test y .solucion
   *******************************************************************/
void GenerarTest(Lista & l)
{
  int num_max_veces, num_preg, num_preg_lista;
  Lista candidatas, test;
  Valor p;
  bool fin;
  char c;


  if (l.ListaVacia())
     cerr << "La lista de preguntas esta vacia" << endl;
  else
  {
      cout << "Introduzca numero de veces de repeticion maxima de una pregunta: ";
      cin >> num_max_veces;

      // Recorremos la lista l ('almacen') y copiamos en 'candidatas' aquellas preguntas que
      // no superen el numero de repeticiones permitido
      l.IrInicio();
      num_preg_lista = 0;   // Numero de preguntas de la lista 'candidatas'
      while (l.Consultar(p))
      {
         if (p.veces <= num_max_veces)
         {
            num_preg_lista ++;
            candidatas.Insertar(p);
            l.Avanzar();
         }
      }

      fin = false;
      num_preg = 0;

      if (num_preg_lista == 0)
      {
         cout << "Segun el criterio seleccionado no quedan preguntas para generar el test" << endl;
         fin = true;
      }
      else
      {
         do //introducir el num. de preguntas del test
         {
            cout << "Introduzca numero de preguntas del test (maximo " << num_preg_lista <<"): ";
            cin >> num_preg;

            if (num_preg > num_preg_lista)
            {
               cout << "El numero de preguntas es superior a las preguntas disponibles" << endl;
               do {
                     cout << "Pulse 'S' para introducir de nuevo el numero de preguntas o 'N' para salir al menu" << endl;
                     c = cin.get(); cin.ignore();
               }
               while ((c != 'S') && (c != 'N'));
               if (c == 'N') fin = true;
            }
         }
         while ((!fin) && ((num_preg <= 0) || (num_preg > num_preg_lista)));

         if (!fin)
         {
            // Seleccionamos las preguntas de la lista 'candidatas'
            candidatas.IrInicio();
            while (num_preg > 0)
            {
                // Realizamos movimiento aleatorio (movemos punto de inter�)
                MovimientoAleatorio(candidatas,num_preg_lista);

                // Seleccionamos la pregunta
                candidatas.Consultar(p);

                // Sumamos 1 en repetici� de la pregunta en la lista 'l'
                if (BuscaPreguntaPorID(l,p.ID))
                   SumaUnaVez(l);

                // Eliminamos la pregunta de la lista 'candidatas'
                candidatas.Eliminar();

                // Insertamos la pregunta en la lista 'test'
                test.Insertar(p);

                num_preg--;
            }

            cout << "El test ha sido generado correctamente" << endl;
            // Escribimos test a ficheros (.test y .solucion)
            EscribeTest(test);
          }
      }
   }
}



/******************************************************************
  Realiza un movimiento aleatorio del punto de inter� en la lista
  pasada como par�etro, num_max es el numero de elementos de la lista
  e indica la distancia maxima del movimiento a realizar
*******************************************************************/
void MovimientoAleatorio(Lista & l, int num_max)
{
   int aleat_sentido, aleat_saltos;
   Valor p;

   // Generamos el sentido del movimiento y el tamañño del salto
   aleat_sentido = rand()%2;
   aleat_saltos = rand()%num_max;

   // REALIZAMOS EL MOVIMIENTO EN LA LISTA
   if (aleat_sentido == 0)
      // Movimiento hacia adelante
     while (aleat_saltos > 0)
     {
         aleat_saltos--;
         //Avanzar en la lista, pero si se llega al nodo cabeza
         // ir al inicio porque la cabeza no cuenta como elemento
         if (!l.Avanzar ()) 
            l.IrInicio();
     }
   else
      // Movimiento hacia atras
      while (aleat_saltos > 0)
      {
         aleat_saltos--;
         //Retroceder en la lista, pero si se llega al nodo inicio
         //hay que ir al ultimo
         if (!l.Retroceder())
            l.IrUltimo();
      }
}

/******************************************************************************/
/* IntroducePregunta     permite introducir los datos de una nueva pregunta   */
/* parametros E/S                                                             */
/* f          S     los datos de la nueva pregunta introducida                */
/******************************************************************************/
void IntroducePregunta(Lista & l)
{
   Valor preg, aux;

   system("cls");
   cout << "Introducir datos de la pregunta \n" << endl;
   cout << "Texto de la Pregunta: \n";
   getline(cin, preg.pregunta);
   cout << "Respuesta A: \n";
   getline(cin, preg.resA);
   cout << "Respuesta B: \n";
   getline(cin, preg.resB);
   cout << "Respuesta C: \n";
   getline(cin, preg.resC);

   do
   {
      cout << "Respuesta correcta (A,B o C): \n";
      cin >> preg.correcta;
   } while ((preg.correcta < 'A') || (preg.correcta > 'C'));

   do
   {
      cout << "Nivel de dificultad (1-5): \n";
      cin >> preg.nivel;
   } while((preg.nivel < 1) || (preg.nivel > 5));

   preg.veces = 0; //Inicialmente se pone a cero porque no ha sido usada nunca

   // Buscamos ltimo ID para sumarle 1
   if (l.ListaVacia())
      preg.ID = 1;
   else
   {
       l.IrUltimo();
       l.Consultar(aux);
       preg.ID = aux.ID + 1;
       // Situamos punto de interes en la cabeza
       l.Avanzar();
   }

   // Insertamos al final de la lista
   l.Insertar(preg);
}

/* ****************************************************************
   Se pide al usuario el identificador (ID) de pregunta a eliminar y
   se muestra por pantalla, en caso de aceptar el usuario se elimina
   la pregunta
   *****************************************************************/
void EliminaPregunta(Lista &l)
{
   int ID;
   Valor p;
   char c;

   if (l.ListaVacia())
   {
      cout << "La lista de preguntas esta vacia" << endl;
   }
   else
   {
       cout << "Introduzca identificador de la pregunta a eliminar: ";
       cin >> ID;

       // Situa el punto de inter� en la pregunta con cierto ID
       if (BuscaPreguntaPorID(l,ID))
       {
          l.Consultar(p);
          cout << "Si es esta la pregunta a eliminar pulse 'S', en caso contrario 'N'"<< endl;
          cout << "*****************************************"<< endl;
          MuestraPregunta(p);
          cout << "*****************************************"<< endl << "'S' o 'N': ";
          c = cin.get(); cin.ignore();
          if ((c == 'S') || (c == 's'))
             if (l.Eliminar())
                 cout << "Pregunta eliminada correctamente" << endl;
             else
                 cout << "La pregunta no ha podido ser borrada" << endl;
          else
             cout << "La pregunta no ha sido borrada" << endl;
       }
       else
           cout << "La pregunta no se encuentra en la lista." << endl;

   }
}


/******************************************************************************/
/* MostrarLista     muestra la lista de preguntas                             */
/* parametros E/S                                                             */
/* l      S     lista de preguntas a mostrar                            */
/******************************************************************************/
void MostrarLista(Lista l)
{
   Valor t;
   int i = 1;

   if (l.ListaVacia())
      cout << "La lista no contiene preguntas" << endl;
   else
   {
       l.IrInicio();
       while(!l.FinalLista())
       {
           l.Consultar(t);
           cout << i << ") ";
           MuestraPregunta(t);

           l.Avanzar();
           i++;
       }
   }
}

/******************************************************************************/
/* EscribeFich     escribe la lista de preguntas disponibles en el fichero    */
/* parametros E/S                                                             */
/* l      S     lista de preguntas a escribir                           */
/******************************************************************************/
void EscribeFich(Lista  &l)
{
   ofstream fich;
   Valor t;
   bool continua;
   
   if(!fich)
       cerr << "Error de apertura de fichero\n";
   else
   {
       if (l.ListaVacia())
          cout << "La lista de preguntas esta vacia" << endl;
       else
       {
           fich.open("preguntas.txt");
           l.IrInicio();
           continua = true;
           while (continua)
           {
                 if (l.Consultar(t))
                 {
                    fich << t.pregunta << endl;
                    fich << t.resA << endl;
                    fich << t.resB << endl;
                    fich << t.resC << endl;
                    fich << t.correcta << endl;
                    fich << t.nivel << endl;
                    fich << t.veces << endl;
                    continua = l.Avanzar();
                 }
                 else continua = false;
           }
           fich.close();
           cout << "Las preguntas has sido escritas en el fichero" << endl;
       }
   }
}

/******************************************************************************/
/* EscribeTest     escribe la lista del test en el fichero                    */
/* parametros E/S                                                             */
/* l      S     lista de preguntas a escribir                           */
/******************************************************************************/
void EscribeTest(Lista &l)
{
   ofstream fich,fich2;
   Valor t;
   string nombre, F_Test, F_Solucion;
   int cont;

   cout << "Introduzca nombre del fichero de salida: ";
   cin >> nombre;
   F_Test = nombre + ".test";
   F_Solucion = nombre + ".solucion";

   fich.open(F_Test.c_str());
   if(!fich)
       cerr << "Error de apertura de fichero de test\n";
   else
   {
       fich2.open(F_Solucion.c_str());
       if(!fich2)
              cerr << "Error de apertura de fichero de solucion\n";
       else
       {
              l.IrInicio();
              cont = 1; //numera las preguntas
              while(!l.FinalLista())
              {
                         l.Consultar(t);
                         fich << cont << ".- " << t.pregunta << endl;
                         fich << "  (a) " << t.resA << endl;
                         fich << "  (b) " <<  t.resB << endl;
                         fich << "  (c) " <<  t.resC << endl;
                         fich << "---------------------------------------\n";
                         fich2 <<  t.correcta << endl;
                         fich2 << "---------------------------------------\n";
                         l.Avanzar();
                         cont++;
              }
              fich2.close();    
              cout << "Se ha generado correctamente el fichero el test y la solucion." << endl;
       }    
       fich.close();
   }
}

/******************************************************************************/
/* Muestra por pantalla los datos de una pregunta pasada como par�etro       */
/******************************************************************************/
void MuestraPregunta (Valor f)
{
   cout.flush();
   cout << "Texto Pregunta: " << endl;
   cout <<  f.pregunta << endl;
   cout << "Opcion A: ";
   cout <<  f.resA << endl;
   cout << "Opcion B: ";
   cout <<  f.resB << endl;
   cout << "Opcion C: ";
   cout <<  f.resC << endl;
   cout << "SOLUCION: ";
   cout <<  f.correcta << endl;
   cout << "Numero de veces usada en examenes: ";
   cout <<  f.veces << endl;
   cout << "Nivel de dificultad (1..5): ";
   cout <<  f.nivel << endl;
   cout << "Identificador de pregunta: ";
   cout <<  f.ID << endl;
   cout << "------------------------------------------\n";
}

/******************************************************************************/
/* Halla la media de dificultad de las preguntas de la lista                  */
/******************************************************************************/
void HallaMediaDif (Lista l)
{
   int num_preg = 0;
   float acumulador = 0;
   Valor t;

   l.IrInicio();
   while(!l.FinalLista())
   {
       l.Consultar(t);
       acumulador = acumulador + t.nivel;
       num_preg++;
       l.Avanzar();
   }

   cout << "**********************************************************************" << endl;
   if (num_preg > 0)
      cout << "Para un total de " << num_preg << " preguntas, la media de dificultad es " << acumulador / num_preg << endl;
   else
       cout << "No hay preguntas" << endl;
   cout << "**********************************************************************" << endl;
}

/*********************************************************************
  Halla el nmero de preguntas usadas alguna vez en algn test
**********************************************************************/
void HallaNumPregUsadas (Lista l)
{
   Valor t;
   int total = 0;
   int usadas = 0;

   l.IrInicio();
   while(!l.FinalLista())
   {
       l.Consultar(t);
       if(t.veces > 0)
            usadas++;
       total++;
       l.Avanzar();
   }
   cout << "**********************************************************************" << endl;
   cout << "De un total de " << total << " preguntas, " << usadas << " han sido usadas alguna vez" << endl;
   cout << "**********************************************************************" << endl;
}

/*********************************************************************
  Halla el nmero de preguntas no usadas nunca
**********************************************************************/
void HallaNumPregNoUsadas(Lista l)
{
   Valor t;
   int total = 0;
   int Nousadas = 0;

   l.IrInicio();
   while(!l.FinalLista())
   {
       l.Consultar(t);
       if(t.veces == 0)
            Nousadas++;
       total++;
       l.Avanzar();
   }
   cout << "**********************************************************************" << endl;
   cout << "De un total de " << total << " preguntas, " << Nousadas << " no han sido nunca usadas" << endl;
   cout << "**********************************************************************" << endl;
}

/******************************************************************************/
/* MostrarLista     muestra la lista de preguntas de un nivel determinado     */
/* parametros E/S                                                             */
/* l      S     lista de preguntas a mostrar                            */
/******************************************************************************/
void MostrarPreguntasPorNivel(Lista l)
{
   Valor t;
   int num;
   int total = 0;
   
   do
   {
      cout << "Nivel de dificultad de las preguntas a mostrar (1-5): \n";
      cin >> num;
   } while((num < 1) || (num > 5));
   
   l.IrInicio();
   while(!l.FinalLista())
   {
       l.Consultar(t);
       if(t.nivel == num)
       {
            total++;
            MuestraPregunta(t);
       }   
       l.Avanzar();
   }
   cout << "**********************************************************************" << endl;
   cout << "Total preguntas mostradas de nivel " << num << " = " << total << endl;
   cout << "**********************************************************************" << endl;
}

/*******************************************************************
  Situa el punto de interes en la pregunta con ID pasado como parametro
  Devuelve false si no encuentra la pregunta con dicho ID
  ******************************************************************/
bool BuscaPreguntaPorID (Lista& l, int id)
{
     bool res, avanza;
     Valor p;

     res = false;

     if (!l.ListaVacia())
     {
         l.IrInicio();
         while (l.Consultar(p) && (p.ID != id))
            l.Avanzar();

         if (p.ID == id)
            res = true;
     }
     return res;
}

// Suma 1 al campo 'veces' a la pregunta que apunta el punto de interes
// devuelve falso si el punto de interes esta fuera de la lista
bool SumaUnaVez (Lista& l)
{
      Valor p;
      bool res = true;

      if (l.Consultar(p))
      {
         p.veces = p.veces + 1;
         l.Eliminar();
         l.Insertar(p);
      }
      else res = false;

      return res;
}
